﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.
	
//	 This function operates on an open single image graph.
	
	Function Change_Lightness()
		SetDataFolder root:
			String/G G_Image 
			String Image = G_Image, ImageGraphName
			If( ItemsInList( ImageNameList( "", ";" ) ) > 1 ) // Check to be sure the top graph has only a single image
				DoAlert 1, "There are multiple images in the top graph. This function only processes 1 of them.\r Check to be sure the correct graph is the one that is active.\r\rDo you want to continue?"
				Print "There are multiple images in the top graph. This function only processes 1 of them."
				AbortOnValue V_Flag == 2, 0
			EndIf
			If( ItemsInList( ImageNameList( "", ";" ) ) < 1 ) // Check to be sure the top graph has an image
				ABORT "There are no images in the top graph. Check to be sure the correct graph is the one that is active."
				Print "There are no images in the top graph. Check to be sure the correct graph is the one that is active."
				AbortOnValue V_Flag == 2, 0
			EndIf
			Image = StringFromList( 0, ImageNameList( "", ";" ) ) // Image is assigned the name of the image in the top graph
			G_Image = Image
			ImageGraphName = WinName( 0, 1, 1 ) // Saves the name of the active graph so that it can be moved back to the front when the function finishes
			If( StrSearch( Image, "_LA", 0 ) != -1 ) // Checks to see if the active image has already been adjusted
				DoAlert/T = "Adjusted Image Warning" 1, "The active image is an adjusted image. Do you want to make a second adjustment?"
				AbortOnValue V_Flag == 2, 0
			EndIf
			If( StringMatch( Image, "M_HSL2RGB" ) ) // Checks to see if the active image has already been adjusted
				ABORT "The active image is the default transform output \"M_HSL2RGB\".\r\r Don't process this one."
			EndIf
			WAVE ImageWv = ImageNameToWaveRef( "", Image ) // Sets the wave reference for the image
			If( WaveType( ImageWv ) != 72 )
				Print "The active image is not in the unsigned byte format. Use the Redimension command to change it."
				ABORT "The active image is not in the unsigned byte format. Use the Redimension command to change it."
			EndIf
			ImageTransform rgb2hsl, ImageWv // Create HSL version of image
			Make/O/N = ( DimSize( ImageWv, 0 ), DimSize( ImageWv, 1 ) ) Mask // Create 2D Mask for L layer
			WAVE M_RGB2HSL // Default name of HSL wave
			Mask = M_RGB2HSL[p][q][2] // Set Mask equal to L layer
			Histogram/B = { 0, 1, 255 } Mask // Create a histogram of L values
			WAVE W_Histogram // Default output of Histogram
			Integrate W_Histogram // Integrate histogram curve to make a cumulative curve
			W_Histogram /= WaveMax( W_Histogram ) * .01 // Normalize cumulative to 0-100%
			Variable TwoPerCent, NinetyEightPerCent
			FindLevel/Q W_Histogram, 2 // Finds the cumulative level ( 0 - 255 ) of 2% of the lightness values
			TwoPerCent = V_LevelX 
			FindLevel/Q W_Histogram, 98 // Finds the cumulative level ( 0 - 255 ) of 98% of the lightness values
			NinetyEightPerCent = V_LevelX 
			DoWindow/K HistogramPlot // Kills histogram window if it is open
			Display/W=(486,69,1467,528)/N = HistogramPlot W_Histogram as "HistogramPlot" // New histogram window
			ModifyGraph grid=1,nticks=10,minor=1,gridStyle=1
			DoUpdate // Updates Histogram window
			String/G G_Process 
			String Process = G_Process
			Prompt Process, "Choose Process", popup, "DC Offset;Multiplier;Histogram Stretch" // List of process choices
			DoPrompt "Change_Lightness", Process // Selects which process to use on the image
			G_Process = Process
			AbortOnValue V_Flag, 0
			DoWindow/F $ImageGraphName // Moves the original image graph back to the front
			StrSwitch( Process )
				Case "DC Offset": // DCOffset is a constant offset of the lightness values
					Variable/G G_DCOffset
					Variable DCOffset = G_DCOffset
					Prompt DCOffset, "DC Offset?"
					DoPrompt "Change_Lightness", DCOffset
					G_DCOffset = DCOffset
					AbortOnValue V_Flag, 0
					Change_Lightness_F( Process, DCOffset) // Runs the _F function for the calculation
					Print "Change_Lightness_F( \"" + Process + "\", " + Num2Str( DCOffset ) + " )"
					Break
				Case "Multiplier":
					Variable/G G_Multiplier
					Variable Multiplier = G_Multiplier
					Prompt Multiplier, "Multiplier?"
					DoPrompt "Change_Lightness", Multiplier
					G_Multiplier = Multiplier
					AbortOnValue V_Flag, 0
					Change_Lightness_F( Process, Multiplier) // Runs the _F function for the calculation
					Break
				Case "Histogram Stretch": // Runs the _F function for the calculation
					Change_Lightness_F( Process, 1 ) // the "1" is a place holder and has no siginificance for the stretch process
					Break
			EndSwitch
	End				
	
	Function Change_Lightness_F( Process, OffsetOrMultiplier )
		String Process
		Variable OffsetOrMultiplier
		SetDataFolder root:
			String/G G_Image 
			String Image, ImageGraphName
			If( StrSearch( "DC Offset;Multiplier;Histogram Stretch", Process, 0, 2 ) == -1 ) // Checks to see that the Process is one of the choices used in the StrSwitch below 
				Print Process + " is not one of the Process options. They are:\r\t\"DC Offset\"\r\t\"Multiplier\"\r\t\"Histogram Stretch\"\r\tPerhaps a spelling error?"
				ABORT Process + " is not one of the Process options. They are:\r\r\"DC Offset\"\r\"Multiplier\"\r\"Histogram Stretch\"\r\rPerhaps a spelling error?"
			EndIf
			If( ItemsInList( ImageNameList( "", ";" ) ) > 1 ) // Check to be sure the top graph has only a single image
				DoAlert 1, "There are multiple images in the top graph. This function only processes 1 of them.\r Check to be sure the correct graph is the one that is active.\r\rDo you want to continue?"
				Print "There are multiple images in the top graph. This function only processes 1 of them."
				AbortOnValue V_Flag == 2, 0
			EndIf
			If( ItemsInList( ImageNameList( "", ";" ) ) < 1 ) // Check to be sure the top graph has a single image
				Print "There are no images in the top graph. Check to be sure the correct graph is the one that is active."
				ABORT "There are no images in the top graph. Check to be sure the correct graph is the one that is active."
				AbortOnValue V_Flag == 2, 0
			EndIf
			Image = StringFromList( 0, ImageNameList( "", ";" ) ) // Image is assigned the name of the image in the top graph
			G_Image = Image
			ImageGraphName = WinName( 0, 1, 1 ) // Saves the name of the active graph so that it can be moved back to the front when the function finishes
			If( StrSearch( Image, "_LA", 0 ) != -1 ) // Checks to see if the active image has already been adjusted
				DoAlert/T = "Adjusted Image Warning" 1, "The active image is an adjusted image. Do you want to make a second adjustment?"
				AbortOnValue V_Flag == 2, 0
			EndIf
			If( StringMatch( Image, "M_HSL2RGB" ) ) // Checks to see if the active image has already been adjusted
				ABORT "The active image is the default transform output \"M_HSL2RGB\".\r\r Don't process this one."
			EndIf
			WAVE ImageWv = ImageNameToWaveRef( "", Image ) // Sets the wave reference for the image
			If( WaveType( ImageWv ) != 72 )
				Print "The active image is not in the unsigned byte format. Use the Redimension command to change it."
				ABORT "The active image is not in the unsigned byte format. Use the Redimension command to change it."
			EndIf
			ImageTransform rgb2hsl, ImageWv // Create HSL version of image
			Make/O/N = ( DimSize( ImageWv, 0 ), DimSize( ImageWv, 1 ) ) Mask // Create 2D Mask for L layer
			WAVE M_RGB2HSL // Default name of HSL wave
			Mask = M_RGB2HSL[p][q][2] // Set Mask equal to L layer
			Histogram/B = { 0, 1, 255 } Mask // Create a histogram of L values
			WAVE W_Histogram // Default output of Histogram
			Integrate W_Histogram // Integrate histogram curve to make a cumulative curve
			W_Histogram /= WaveMax( W_Histogram ) * .01 // Normalize cumulative to 0-100%
			Variable TwoPerCent, NinetyEightPerCent
			FindLevel/Q W_Histogram, 2 // Finds the cumulative level ( 0 - 255 ) of 2% of the lightness values
			TwoPerCent = V_LevelX 
			FindLevel/Q W_Histogram, 98 // Finds the cumulative level ( 0 - 255 ) of 98% of the lightness values
			NinetyEightPerCent = V_LevelX 
			Print TwoPerCent,"@ 2%\t\t", NinetyEightPerCent, "@ 98%" 
			DoWindow/K HistogramPlot // Kills histogram window if it is open
			Display/W=(486,69,1467,528)/N = HistogramPlot W_Histogram as "HistogramPlot" // New histogram window
			ModifyGraph grid=1,nticks=10,minor=1,gridStyle=1
			DoUpdate // Updates Histogram window
			StrSwitch( Process )
				Case "DC Offset": // DCOffset is a constant offset of the lightness values
					Variable/G G_DCOffset = OffsetOrMultiplier
					Variable DCOffset = G_DCOffset
					Mask += DCOffset // Offset is added to lightness
					Break
				Case "Multiplier":
					Variable/G G_Multiplier = OffsetOrMultiplier
					Variable Multiplier = G_Multiplier
					If( Multiplier < 0 )
						Print "Multiplier cannot be negative. Images are unsigned byte format."
						ABORT "Multiplier cannot be negative. Images are unsigned byte format."
					EndIf
					Mask *= Multiplier // Lightness is multiplied by a constant
					Mask -= ( TwoPerCent * Multiplier ) - TwoPerCent // Lightness is readjusted so that the darker areas are the same as the original image
					Break
				Case "Histogram Stretch":
					OffsetOrMultiplier = 1
					Mask -= TwoPerCent // Shifts 2% lightness value to 0
					Mask *= 255 / ( NinetyEightPerCent - TwoPerCent ) // Adjusts 2-98% to 0-255
					Break
			EndSwitch
			Mask = Mask > 255 ? 255 : Mask // Sets lightness values > 255 to 255
			Mask = Mask < 0 ? 0 : Mask // Sets lightness values less than 0 to 0
			Histogram/B = { 0, 1, 255 } Mask // New histogram of adjusted L values
			Integrate W_Histogram // Integrate histogram curve to make a cumulative curve
			W_Histogram /= WaveMax( W_Histogram ) * .01 // Normalize cumulative to 0-100%
			M_RGB2HSL[][][2] = Mask[p][q] // Replace L layer with Mask
			ImageTransform hsl2rgb M_RGB2HSL // Go back to RGB space
			WAVE M_HSL2RGB
			M_HSL2RGB /= 257 // Conversion for unsigned word to unsigned byte
			Redimension/B/U M_HSL2RGB // 2nd step of conversion
			Duplicate/O M_HSL2RGB $Image + "_LA" // Adjusted image assigned name of original plus "_LA"
			WAVE AdjImg = $Image + "_LA"
			Note AdjImg, note( ImageWv ) + " Change_Lightness_F( \"" + Process + "\", " + Num2Str( OffsetOrMultiplier ) + " )" // Adds wavenote with command line process
			Print "Change_Lightness_F( \"" + Process + "\", " + Num2Str( OffsetOrMultiplier ) + " )" // Adds wavenote with command line process
			If( ItemsInList( WinList( "AdjustedImagePlot", ";", "Visible:1" ) ) == 0 ) // Checks to see if the adjusted image plot is already open
				Display/W=(486,69,1467,528)/N = AdjustedImagePlot as "M_HSL2RGB"; AppendImage M_HSL2RGB // New adjusted image window
			EndIf
			DoWindow/F $ImageGraphName // Moves the original image graph back to the front
			KillWaves/Z Mask, M_RGB2HSL
	End
	
	
	Function Batch_Change_Lightness(  )
		SetDataFolder root:
			String/G G_Process, G_Folder
			Variable/G G_OffsetOrMultiplier
			String Process = G_Process, Folder = G_Folder
			Variable OffsetOrMultiplier = G_OffsetOrMultiplier, i
			String/G G_Image 
			String Image, ImageList			
			Prompt Folder, "Folder containing images to process:", popup, "root:;" + FolderList("*")
			Prompt Process, "Process:", popup, "DC Offset;Multiplier;Histogram Stretch"
			Prompt OffsetOrMultiplier, "Offset or Multiplier:"
			DoPrompt "Batch_Change_Lightness", Folder, Process, OffsetOrMultiplier
			G_Folder = Folder, G_Process = Process, G_OffsetOrMultiplier = OffsetOrMultiplier
			AbortOnValue V_Flag, 0
			String NewDF = "root:" + Folder + "_LA"
			NewDataFolder/O $NewDF
			NewDF = NewDF + ":"
		SetDataFolder $Folder
			KillWaves/Z Mask, M_RGB2HSL, M_HSL2RGB, W_Histogram
			ImageList = SortList( WaveList( "*", ";", "DIMS:3 " ) )
			For( i = 0; i < ItemsInList( ImageList ); i += 1 )
				Image = StringFromList( i, ImageList ) // Image is assigned the name of the image in the top graph
				Printf "%s, ", Image
				WAVE ImageWv = $Image // Sets the wave reference for the image
				If( WaveType( ImageWv ) != 72 )
					Print Image + " is not in the unsigned byte format. Use the Redimension command to change it."
					ABORT Image + " is not in the unsigned byte format. Use the Redimension command to change it."
				EndIf
				ImageTransform rgb2hsl, ImageWv // Create HSL version of image
				Make/O/N = ( DimSize( ImageWv, 0 ), DimSize( ImageWv, 1 ) ) Mask // Create 2D Mask for L layer
				WAVE M_RGB2HSL // Default name of HSL wave
				Mask = M_RGB2HSL[p][q][2] // Set Mask equal to L layer
				Histogram/B = { 0, 1, 255 } Mask // Create a histogram of L values
				WAVE W_Histogram // Default output of Histogram
				Integrate W_Histogram // Integrate histogram curve to make a cumulative curve
				W_Histogram /= WaveMax( W_Histogram ) * .01 // Normalize cumulative to 0-100%
				Variable TwoPerCent, NinetyEightPerCent
				FindLevel/Q W_Histogram, 2 // Finds the cumulative level ( 0 - 255 ) of 2% of the lightness values
				TwoPerCent = V_LevelX 
				FindLevel/Q W_Histogram, 98 // Finds the cumulative level ( 0 - 255 ) of 98% of the lightness values
				NinetyEightPerCent = V_LevelX 
				StrSwitch( Process )
					Case "DC Offset": // DCOffset is a constant offset of the lightness values
						Variable/G G_DCOffset = OffsetOrMultiplier
						Variable DCOffset = G_DCOffset
						Mask += DCOffset // Offset is added to lightness
						Break
					Case "Multiplier":
						Variable/G G_Multiplier = OffsetOrMultiplier
						Variable Multiplier = G_Multiplier
						If( Multiplier < 0 )
							Print "Multiplier cannot be negative. Images are unsigned byte format."
							ABORT "Multiplier cannot be negative. Images are unsigned byte format."
						EndIf
						Mask *= Multiplier // Lightness is multiplied by a constant
						Mask -= ( TwoPerCent * Multiplier ) - TwoPerCent // Lightness is readjusted so that the darker areas are the same as the original image
						Break
					Case "Histogram Stretch":
						OffsetOrMultiplier = 1
						Mask -= TwoPerCent // Shifts 2% lightness value to 0
						Mask *= 255 / ( NinetyEightPerCent - TwoPerCent ) // Adjusts 2-98% to 0-255
						Break
				EndSwitch
				Mask = Mask > 255 ? 255 : Mask // Sets lightness values > 255 to 255
				Mask = Mask < 0 ? 0 : Mask // Sets lightness values less than 0 to 0
				Histogram/B = { 0, 1, 255 } Mask // New histogram of adjusted L values
				Integrate W_Histogram // Integrate histogram curve to make a cumulative curve
				W_Histogram /= WaveMax( W_Histogram ) * .01 // Normalize cumulative to 0-100%
				M_RGB2HSL[][][2] = Mask[p][q] // Replace L layer with Mask
				ImageTransform hsl2rgb M_RGB2HSL // Go back to RGB space
				WAVE M_HSL2RGB
				M_HSL2RGB /= 257 // Conversion for unsigned word to unsigned byte
				Redimension/B/U M_HSL2RGB // 2nd step of conversion
				Duplicate/O M_HSL2RGB $NewDF + Image + "_LA" // Adjusted image assigned name of original plus "_LA"
				WAVE AdjImg = $NewDF + Image + "_LA"
//				Smooth/M = 10, 50, AdjImg
				Note AdjImg, note( ImageWv ) + " Change_Lightness_F( \"" + Process + "\", " + Num2Str( OffsetOrMultiplier ) + " )" // Adds wavenote with command line process
			EndFor
			Print "\r"
			KillWaves/Z Mask, M_RGB2HSL, M_HSL2RGB, W_Histogram
		SetDataFolder root:			
	End
	
