﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

	
	Function CaCO3BatchLoad()
		SetDataFolder root:
			KillPath/Z Data_Path
			KillVariables/Z Hole
			String DataFileList
			Variable/G G_CC_Replacement
			Variable CC_Replacement = G_CC_Replacement, i
			Prompt CC_Replacement, "Replace CC's with column number used in Coring Matrix ( 9 is usual )" 
			DoPrompt "CaCO3BatchLoad", CC_Replacement
			G_CC_Replacement = CC_Replacement; AbortOnValue V_Flag, 0
			DoAlert 0, "Select the data folder.\r\rThis function will attempt to process all \".csv\" files in the selected folder. You will generate an error if there are non-CaCO3 files."
			NewPath Data_Path
			DataFileList = IndexedFile( Data_Path, -1, ".csv" )
			For( i = 0; i < ItemsInList( DataFileList ); i += 1 )
				LoadWave/O/A/P=Data_Path/Q/J/W/K=0/L={0,1,0,0,0}/v={",","\"",0,0} StringFromList( i, DataFileList )				
				WAVE/T Site,Hole,Type
	  			WAVE ExpW,Core,Sect,Top_offset_on_section__cm_,Bot_offset_on_section__cm_,Top_depth_CSF_A__m_
	  			WAVE Top_depth_CSF_B__m_,Inorganic_carbon__wt__,Calcium_carbonate__wt__
				If( StrSearch( Site[ 0 ], "\"", 0 ) > -1 ) // This section takes care of the quotes in the downloaded data file if they are present
					String S2 = Site[0]; S2 = S2[ 1 , StrLen( S2 ) - 2 ]; Site = S2
					String H2 = Hole[0]; H2 = H2[ 1 , StrLen( H2 ) - 2 ]; Hole = H2
					String T2 = Type[0]; T2 = T2[ 1 , StrLen( T2 ) - 2 ]; Type = T2
				EndIf
				String H = Site[0] + Hole[0], SiteNum = RemoveEnding( H )
				Replace_NaNswithNumber_F( "Sect", CC_Replacement )
				NewDataFolder/O $SiteNum + "_CaCO3"
				Duplicate/O Site, $":" + SiteNum + "_CaCO3:" + H + "_CaCO3_Site"; 
				Duplicate/O Hole, $":" + SiteNum + "_CaCO3:" + H + "_CaCO3_Hole"; 
				Duplicate/O Core, $":" + SiteNum + "_CaCO3:" + H + "_CaCO3_Core"; 
				Duplicate/O Type, $":" + SiteNum + "_CaCO3:" + H + "_CaCO3_Type"; 
				Duplicate/O Sect, $":" + SiteNum + "_CaCO3:" + H + "_CaCO3_Section"; 
			Duplicate/O Top_offset_on_section__cm_, $":" + SiteNum + "_CaCO3:" + H + "_CaCO3_Interval"; 
			Duplicate/O Top_depth_CSF_A__m_, $":" + SiteNum + "_CaCO3:" + H + "_CaCO3_CSFA"; 
			Duplicate/O Inorganic_carbon__wt__, $":" + SiteNum + "_CaCO3:" + H + "_CaCO3_InorgC"; 
			Duplicate/O Calcium_carbonate__wt__, $":" + SiteNum + "_CaCO3:" + H + "_CaCO3_CaCO3"
			// There are more waves listed below to be killed than recognized in WAVE statements above.
			// This is because LORE has 2 different formats for CaCO3 data. Notice that /Z is used for KillWaves for times when some
			//		of these waves are not in the data file and so not loaded.
			KillWaves/Z Site,Hole,Type,A_W,Sample_treatment_method__CHNS_o
  			KillWaves/Z ExpW,Core,Sect,Top_offset_on_section__cm_,Bot_offset_on_section__cm_,Top_depth_CSF_A__m_
  			KillWaves/Z Top_depth_CSF_B__m_,Inorganic_carbon__wt__,Calcium_carbonate__wt__,Total_carbon__wt__,Hydrogen__wt__
  			KillWaves/Z Nitrogen__wt__,Sulfur__wt__,Organic_carbon__wt____CHNS_with,Organic_carbon__wt___by_differe,Comments
			EndFor
	End
			

//	Function CaCO3LOREDataLoader( )
		SetDataFolder root:
			String/G G_H
			String H = G_H, SiteNum
			Variable/G G_CC_Replacement
			Variable CC_Replacement = G_CC_Replacement
			Prompt H, "Hole ( e.g. U1242A )"
			Prompt CC_Replacement, "Replace CC's with column number used in Coring Matrix ( 9 is default )"
			DoPrompt "CaCO3LOREDataLoader", H, CC_Replacement
			G_H = H; G_CC_Replacement = CC_Replacement; AbortOnValue V_Flag, 0
			Print "CaCO3LOREDataLoader_F( \"" + H + "\",", CC_Replacement, ")"
			CaCO3LOREDataLoader_F( H, CC_Replacement )
	End
	
//	Function CaCO3LOREDataLoader_F( H, CC_Replacement )
			String H
			Variable CC_Replacement
			H = UpperStr( H )
			CheckHole4aU( H )
		SetDataFolder root:
			String SiteNum
			SiteNum = RemoveEnding( H )
			WAVE/T Hole, Type, A_W, Sample_treatment_method__CHNS_o, Site,Comments
  			WAVE ExpW,Core,Sect,Top_offset_on_section__cm_,Bot_offset_on_section__cm_,Top_depth_CSF_A__m_
  			WAVE Top_depth_CSF_B__m_,Inorganic_carbon__wt__,Calcium_carbonate__wt__,Total_carbon__wt__,Hydrogen__wt__,Nitrogen__wt__
  			WAVE Sulfur__wt__,Organic_carbon__wt____CHNS_with,Organic_carbon__wt___by_differe
			If( !WaveExists( Calcium_carbonate__wt__ ) )
				Print "I can't find the datawave \"Calcium_carbonate__wt__\".\r\t\tDid you forget to copy the column label line along with the data?\r\t\t\t\tOR\r\t\tAre you sure the data are CaCO3 data from LORE?"
				ABORT "I can't find the datawave \"Calcium_carbonate__wt__\".\r\rDid you forget to copy the column label line along with the data?\r\rOR\r\rAre you sure the data are CaCO3 data from LORE?"
			EndIf
			If( NumType( Core[0] ) == 2 ) // Checks to see if the first line in the DataTable is blank
				DeletePoints 0,1, Hole, Type, A_W, Sample_treatment_method__CHNS_o, Site
				DeletePoints 0,1, ExpW,Core,Sect,Top_offset_on_section__cm_,Bot_offset_on_section__cm_,Top_depth_CSF_A__m_  
				DeletePoints 0,1, Top_depth_CSF_B__m_,Inorganic_carbon__wt__,Calcium_carbonate__wt__,Total_carbon__wt__,Hydrogen__wt__,Nitrogen__wt__   
				DeletePoints 0,1, Sulfur__wt__,Organic_carbon__wt____CHNS_with,Organic_carbon__wt___by_differe,Comments 
			EndIf
			If( !StringMatch( Hole[1], H[ StrLen( H ) - 1 ] ) ) // Checks to see that the data table has data from the designated Hole
				Print "The Hole in the data column is not the same as the Hole input.\r\t\t" + Site[1] + Hole[1] + " vs " + H 
				ABORT "The Hole in the data column is not the same as the Hole input.\r\r" + Site[1] + Hole[1] + " vs " + H 
			EndIf
			If( !StringMatch( Site[1], SiteNum ) )  // Checks to see that the data table has data from the designated Hole
				Print "The Site Number in the data column \"Site\" is not the same as the Site Number input.\r\t\t" + Site[1] + " vs " + SiteNum
				ABORT "The Site Number in the data column \"Site\" is not the same as the Site Number input.\r\r" + Site[1] + " vs " + SiteNum
			EndIf
			If( StrSearch( WinList( "*", ";", "WIN:2" ), "LOREDataTable", 0 ) < 0  || StrSearch( WaveList( "*", ",", "WIN:LOREDataTable" ), "Calcium_carbonate__wt__", 0 ) < 0 ) // Checks that the data are in the designated Table named "LOREDataTable"
				Print "The data are not in a table named \"LOREDataTable\". \r\t\tKill the data waves you just pasted and Run:\r\t\t\t\tEmptyLOREDataTable()\r\t\tfrom the LORE_Loaders menu and then paste the data into the new blank table.\r\t\tAfter the first run the table will open automatically."
				ABORT "The data are not in a table named \"LOREDataTable\". \r\rKill the data waves you just pasted and Run:\r\r\"EmptyLOREDataTable\"\r\rfrom the LORE_Loaders menu and then paste the data into the new blank table.\r\rAfter the first run the table will open automatically."
			Endif
			Replace_NaNswithNumber_F( "Sect", CC_Replacement )
			Rename Site, $H + "_CaCO3_Site"; 
			Rename Hole, $H + "_CaCO3_Hole"; 
			Rename Core, $H + "_CaCO3_Core"; 
			Rename Type, $H + "_CaCO3_Type"; 
			Rename Sect, $H + "_CaCO3_Section"; 
			Rename Top_offset_on_section__cm_, $H + "_CaCO3_Interval"; 
			Rename Top_depth_CSF_A__m_, $H + "_CaCO3_CSFA"; 
			Rename Inorganic_carbon__wt__, $H + "_CaCO3_InorgC"; 
			Rename Calcium_carbonate__wt__, $H + "_CaCO3_CaCO3"
			Rename Hydrogen__wt__, $H + "_CaCO3_Hydrogen"
			Rename Nitrogen__wt__, $H + "_CaCO3_Nitrogen"
			Rename Organic_carbon__wt___by_differe, $H + "_CaCO3_OrgC_Diff"
			Rename Organic_carbon__wt____CHNS_with, $H + "_CaCO3_OrgC_CHNS"
			Rename Sulfur__wt__, $H + "_CaCO3_Sulfer"
			EmptyLOREDataTable( )
			KillWaves ExpW,A_W,Bot_offset_on_section__cm_,Top_depth_CSF_B__m_,Total_carbon__wt__,Sample_treatment_method__CHNS_o,Comments
			NewDataFolder/O $SiteNum + "_CaCO3"
	End
			