﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

	Function SRM_Process( )
		SetDataFolder root:
			String/G G_Hole
			Variable/G G_DeclCorr
			String Hole = G_Hole
			Variable DeclCorr = G_DeclCorr
			Prompt Hole, "Hole"
			Prompt DeclCorr, "Enter Local Declination Correction"
			DoPrompt "SRM_Process", Hole, DeclCorr
			G_Hole = Hole; G_DeclCorr = DeclCorr
			AbortOnValue V_Flag, 0
			SRM_Process_F( Hole, DeclCorr )
			Print "SRM_Process_F( \"" + Hole + "\", " + Num2Str( DeclCorr ) + " )"
	End

-------------------------------------------------------

	Function SRM_Process_F( Hole, DeclCorr )
		String Hole	
		Variable DeclCorr	
		SRM_LoadData_F( Hole )
		SetDataFolder root:
			String Site = RemoveEnding( Hole )
		SetDataFolder $Site + "_Orientation"
			WAVE MagToolFace = $Hole + "_Orientation_MagToolFace", OrientCoreNum = $Hole + "_Orientation_Core"
		SetDataFolder $"root:" + Site + "_SRM"
			WAVE SiteWv = $Hole + "_SRM_Site"
			WAVE HoleWv = $Hole + "_SRM_Hole"
			WAVE Core = $Hole + "_SRM_Core"
			WAVE Type = $Hole + "_SRM_Type"
			WAVE Section = $Hole + "_SRM_Section"
			WAVE AorW = $Hole + "_SRM_AorW"
			WAVE Interval = $Hole + "_SRM_Interval"
			WAVE CSF = $Hole + "_SRM_CSF"
			WAVE CSFB = $Hole + "_SRM_CSFB"
			WAVE Treatment = $Hole + "_SRM_Treatment"
			WAVE TreatmentVal = $Hole + "_SRM_TreatmentVal"
			WAVE Inclination = $Hole + "_SRM_InclDriftCorr"
			WAVE Declination = $Hole + "_SRM_DeclDriftCorr"
			WAVE Intensity = $Hole + "_SRM_IntenDriftCorr"
			WAVE InclRaw = $Hole + "_SRM_InclRaw"
			WAVE DeclRaw = $Hole + "_SRM_DeclRaw"
			WAVE IntenRaw = $Hole + "_SRM_IntenRaw"
			WAVE MomentX = $Hole + "_SRM_MomentX"
			WAVE MomentY = $Hole + "_SRM_MomentY"
			WAVE MomentZ = $Hole + "_SRM_MomentZ"
			Duplicate/O Declination $Hole + "_SRM_DeclTensorCorr" // Sets up the wave for the tensor-corrected values
			WAVE TensorCorrDecl = $Hole + "_SRM_DeclTensorCorr"
			TensorCorrDecl = NaN
			Variable i
			For( i = 0; i < NumPnts( OrientCoreNum ); i += 1 ) // This loop works through the tensor corrections and applies them to the appropriate cores
				TensorCorrDecl =  Core == OrientCoreNum[ i ] ? Declination + MagToolFace[ i ] : TensorCorrDecl // Unoriented cores will have NaN as the corrected value
			EndFor
			TensorCorrDecl += DeclCorr // This step accounts for the local difference between magnetic and true North
			TensorCorrDecl =  TensorCorrDecl >= 360 ? TensorCorrDecl - 360 : TensorCorrDecl // This corrects for inclinations > 360 after correction
			Differentiate TreatmentVal /D = DIF
			TreatmentVal = Dif == 0 ? TreatmentVal : NaN
			TreatmentVal[ 0 ] = NaN
			WaveStats/Q TreatmentVal			
				// This next step sorts the data by demagnitization step values 0, 10, 15, 20 etc
			Sort { TreatmentVal, Core, Section, Interval } Core, Type, Section, Interval, CSF, CSFB, TreatmentVal, Inclination, Declination, Intensity, InclRaw, DeclRaw, IntenRaw, MomentX, MomentY, MomentZ, TensorCorrDecl
			Redimension/N = ( V_Npnts ) Core, Type, Section, Interval, CSF, CSFB, TreatmentVal, Inclination, Declination, Intensity
			Redimension/N = ( V_Npnts ) InclRaw, DeclRaw, IntenRaw, MomentX, MomentY, MomentZ, TensorCorrDecl, AorW, HoleWv, SiteWv, Treatment, Type
			Make/O/N = 10 DeMagLevels, DeMagPntLocs
			For( i = 0; i < 10; i += 1 ) // This loop records what demag levels were used and where those levels start ( point number ) in the total data set
				FindValue/V = ( i * 5 ) TreatmentVal // levels from 0 to 45 are looked for
				DeMagLevels[ i ] = i * 5	// The demag levels are assumed to be in increments of 5
				DeMagPntLocs[ i ] = V_Value
			EndFor
			DeMagPntLocs = DeMagPntLocs > -1 ? DeMagPntLocs : NaN // If some levels did not exist their values are set to NaN
			DeMagLevels = DeMagPntLocs > -1 ? DeMagLevels : NaN
			Sort DeMagLevels DeMagPntLocs, DeMagLevels // Sorting puts real values at the top of the waves and NaN's at the bottom
			WaveStats/Q DeMagLevels
			DeMagPntLocs[ V_Npnts ] = inf // This value is set so the final demag level data are copied from their first point to the end of the data
			For( i = 0; i < V_Npnts; i += 1 ) // This loop sets up separate folders for each demag level
				NewDataFolder/O $"root:" + Site + "_SRM" + Num2Str( DeMagLevels[ i ] )
			EndFor			
			String WvList = WaveList( "*SRM*", ";", "" ), WvOut // A list of all of the SRM data waves is formed
			Variable j			
			For( i = 0; i < V_Npnts; i += 1 ) // This loop cycles through the demag levels and the inner loop copies the data to the appropriate demag folder
				For( j = 0; j < ItemsInList( WvList ); j += 1 )
					WvOut = "Root:" + Site + "_SRM" +  Num2Str( DeMagLevels[ i ] ) + ":" + Hole + "_SRM" + Num2Str( DeMagLevels[ i ] ) + "_" + DataWaveData( StringFromList( j, WvList ) )
					Duplicate/O/R = ( DeMagPntLocs[ i ], DeMagPntLocs[ i + 1 ] - 1 ) $StringFromList( j, WvList ) $WvOut
				EndFor
			EndFor
			KillWaves/Z DeMagLevels, DeMagPntLocs, Dif
		SetDataFolder root:	
	End

--------------------------------------------------------------------------------	

	Function SRM_LoadData_F( Hole )
		String Hole		
		SetDataFolder root:
			String Site = RemoveEnding( Hole )
		NewDataFolder/O/S $Site + "_SRM"
			LoadWave/K = 0/J/N = SRM 
			WAVE SRM1, SRM2, SRM3, SRM4, SRM5, SRM6, SRM7, SRM8, SRM9, SRM10
			WAVE SRM11, SRM12, SRM13, SRM14, SRM15, SRM16, SRM17, SRM18, SRM19
			WAVE SRM20, SRM21, SRM22
			Duplicate/O SRM1, $Hole + "_SRM_Site"
			Duplicate/O SRM2, $Hole + "_SRM_Hole"
			Duplicate/O SRM3, $Hole + "_SRM_Core"
			Duplicate/O SRM4, $Hole + "_SRM_Type"
			Duplicate/O SRM5, $Hole + "_SRM_Section"
			Duplicate/O SRM6, $Hole + "_SRM_AorW"
			Duplicate/O SRM7, $Hole + "_SRM_Interval"
			Duplicate/O SRM8, $Hole + "_SRM_CSF"
			Duplicate/O SRM9, $Hole + "_SRM_CSFB"
			Duplicate/O SRM10, $Hole + "_SRM_Treatment"
			Duplicate/O SRM11, $Hole + "_SRM_TreatmentVal"
			Duplicate/O SRM12, $Hole + "_SRM_InclDriftCorr"
			Duplicate/O SRM13, $Hole + "_SRM_DeclDriftCorr"
			Duplicate/O SRM14, $Hole + "_SRM_IntenDriftCorr"
			Duplicate/O SRM15, $Hole + "_SRM_InclRaw"
			Duplicate/O SRM16, $Hole + "_SRM_DeclRaw"
			Duplicate/O SRM17, $Hole + "_SRM_IntenRaw"
			Duplicate/O SRM18, $Hole + "_SRM_MomentX"
			Duplicate/O SRM19, $Hole + "_SRM_MomentY"
			Duplicate/O SRM20, $Hole + "_SRM_MomentZ"
			EXECUTE "KillWaves/Z " + WaveList( "SRM*", ",", "" )
			WAVE Section = $Hole + "_SRM_Section"
			Section = NumType( Section ) == 2 ? 9 : Section // This line replaces blanks left by CC's with 9's
		SetDataFolder root:	
	End
