﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

// New version for data splicing. Created MAR2018.
//

	Function SpliceDataV3()
		SetDataFolder root:
			Variable i, j, NumberOfWaves
			String/G G_Site, G_Technique, G_DepthType, G_YorN
			String Site = G_Site,  Technique = G_Technique, DepthType = G_DepthType, YorN = G_YorN
			Prompt Site, "Site"
			Prompt Technique, "Technique"
			Prompt DepthType, "Depth type ( e.g. CCSF, mcd, CCSFs )"
			Prompt YorN, "Print depth and point ranges for each splice segment to History         ( good for troubleshooting problems with data )", popup, "Y;N"
			DoPrompt "SpliceDataV3", Site, Technique, DepthType, YorN;
			G_Site = Site; G_Technique = Technique	; G_DepthType = DepthType; G_YorN = YorN
			AbortOnValue V_Flag, 0
			SpliceDataV3_F( Site, Technique, DepthType, YorN )
			Print "SpliceDataV3_F( \"" + Site + "\", \"" + Technique + "\", \"" + DepthType + "\", \"" + YorN + "\" )"
	End

	Function SpliceDataV3_F( Site, Technique, DepthType, YorN )
			String Site, Technique, DepthType, YorN
		SetDataFolder root:
			String/G G_Site, G_Technique, G_DepthType
			String SplWvName, DataWvName
			Variable i, j, NumberOfWaves//, HoleLetterPosition
			G_Site = Site; G_Technique = Technique	; G_DepthType = DepthType
			DFREF SpliceFldr = $"root:" + Site + "_Splice_Intervals"
			DFREF DataFldr = $"root:" + Site + "_" + Technique
			Make/O/N=0 Mask, Mask2, MaskPtNumber
			Make/T/O/N=0 TMask
		 	If( !DataFolderExists( "root:" + Site + "_Splice_Intervals" ) ) // Check to see if Spliced Intervals folder exists
		 		ABORT "This function uses the \"Splice_Intervals\" form of splice data. Use the Splice_Format_Conversion function to form an Intervals folder from a Ties folder."
		 	EndIf
		 	If( !DataFolderExists( "root:" + Site + "_" + Technique ) ) // Check to see if data folder exists
		 		ABORT "I can't find the data folder \"" + Site + "_" + Technique + "\"."
		 	EndIf
			If( SpliceDataCheck( Site, "Splice_Intervals" ) ) // This is a separate function to check the splice tables.
				Abort
			Endif
			CheckSiteWavesAreText( Site, Technique ) //Checks to see that all Site waves are either TEXT or Numbers.
		SetDataFolder SpliceFldr
			WAVE CCSF_Top, CCSF_Bottom
			WAVE/T Hole_Top, Type_Top
		SetDataFolder DataFldr
			CCSFNaNCheck()  // This function checks to see if there are blanks in the depth waves. They might cause a problem if they are within splice intervals.
			String HoleList = WaveList( "*" + DepthType, ";", "" ), HoleLetter, WvList, WvList2
			For( i = 0; i < ItemsInList( HoleList ); i += 1 ) // This loop checks to see if all of the Holes have the same number of waves
				HoleLetter = StringFromList( i, HoleList )[ Strlen( Site ) ]
				If( !StringMatch( HoleLetter, "S" ) && !StringMatch( HoleLetter, "_" ) ) // Skips existing spliced data when checking
					WvList = WaveList( Site + HoleLetter + "*", ";", "" )
					 Print Site + HoleLetter + " has", ItemsInList( WvList ), "Waves in " + Site + "_" + Technique
					 If( i == 0 )
					 	NumberOfWaves = ItemsInList( WvList )
					 ElseIf( NumberOfWaves != ItemsInList( WvList ) )
						SetDataFolder root:
					 	ABORT "There are different numbers of waves for different Holes in the data folder. All Holes must have the same number and names of waves in order to splice all of the data.\r\r  See HISTORY for more information."
					 EndIf
				EndIf
			EndFor
			String FirstHoleLetter = StringFromList( 0, HoleList )[ Strlen( Site ) ]
			WvList = SortList( WaveList( Site + FirstHoleLetter + "*", ";", "" ) )
			For( i = 0; i < ItemsInList( HoleList ); i += 1 ) // This loop checks to be sure that there are no spelling errors in the wavenames
				HoleLetter = StringFromList( i, HoleList )[ Strlen( Site ) ]
				If( !StringMatch( HoleLetter, "S" ) && !StringMatch( HoleLetter, "_" ) ) // Skips existing spliced data when checking
					WvList2 = SortList( WaveList( Site + HoleLetter + "*", ";", "" ) )
					WvList2 = ReplaceString( Site + HoleLetter, WvList2, Site + FirstHoleLetter )
					If( CmpStr( WvList, WvList2 ) ) // Looks for typos in wave names
						ABORT "WaveName(s) of Hole " + HoleLetter + " data are spelled differently than for Hole " + FirstHoleLetter
					EndIf
				EndIf
			EndFor
			String TextSpliceList = WaveList( Site + FirstHoleLetter + "*", ";", "TEXT:1" ) // These 4 lines create wavename lists for spliced text and numerical waves
			TextSpliceList = ReplaceString( Site + FirstHoleLetter, TextSpliceList, Site + "S" )
			String NumSpliceList = WaveList( Site + FirstHoleLetter + "*", ";", "TEXT:0" )
			NumSpliceList = ReplaceString( Site + FirstHoleLetter, NumSpliceList, Site + "S" )
			For( i = 0; i < ItemsInList( TextSpliceList ); i += 1 ) // These loops create, or zero out existing, spliced text and numerical waves
				Make/T/O/N = 0 $StringFromList( i, TextSpliceList )
			EndFor
			For( i = 0; i < ItemsInList( NumSpliceList ); i += 1 )
				Make/O/N = 0 $StringFromList( i, NumSpliceList )
			EndFor

			For( i = 0; i < NumPnts( Hole_Top ); i += 1 ) // This section works through the Splice Interval table line by line.
				If( StrSearch( NumSpliceList, "_mcd", 0, 2 ) > 0 ) // Checs to see if depth is mcd or CCSF
					WAVE DepthWv = $Site + Hole_Top[ i ] + "_" + Technique + "_mcd"
				Else	
					WAVE DepthWv = $Site + Hole_Top[ i ] + "_" + Technique + "_CCSF"
				EndIf			
				Duplicate/O $Site + Hole_Top[ i ] + "_" + Technique + "_Core", Mask
				Mask = DepthWv >= CCSF_Top[i] && DepthWv < CCSF_Bottom[i] ? 1 : NaN // MASK identifies which lines of data are within the splice interval.
				WaveStats/Q Mask

				For( j = 0; j < ItemsInList( NumSpliceList ); j += 1 ) // This loop fills in the numerical data.
					WAVE SplWv = $StringFromList( j, NumSpliceList )
					SplWvName = StringFromList( j, NumSpliceList )
					SplWvName = ReplaceString("S", SplWvName, Hole_Top[ i ], 1, 1 )
					WAVE DatWv = $SplWvName // The splice wave name has been changed to the appropriate data wave name ( 2 lines above )
					Duplicate/O Mask, Mask2, MaskPtNumber 
					MaskPtNumber = x
 					Mask2 = NaN
 					Mask2 = DatWv * MASK
					Sort {Mask2,MaskPtNumber}, Mask2, MaskPtNumber // Data from interval sorted so NaN's all follow data
					Redimension/N = ( V_Npnts ) Mask2, MaskPtNumber //NaN's deleted
					Sort { MaskPtNumber, Mask2 }, Mask2, MaskPtNumber // Data resorted to original sequence using original point numbers.
 					Concatenate/NP { Mask2 }, SplWv // Interval of data is added to splice wave
				EndFor

				For( j = 0; j < ItemsInList( TextSpliceList ); j += 1 ) // This loop fills in the text data.
					WAVE/T TSplWv = $StringFromList( j, TextSpliceList )
					SplWvName = StringFromList( j, TextSpliceList )
					SplWvName = ReplaceString("S", SplWvName, Hole_Top[ i ], 1, 1 )
					WAVE/T TDatWv = $SplWvName
					Redimension/N = ( V_Npnts ) TMask
					If( StrSearch( NameOfWave( TDatWv ), "Type", 0, 2 ) > 0 )
						TMask = Type_Top[ i ]
					Else
						TMask = TDatWv[ 0 ]
 					EndIf
 					Concatenate/T/NP { TMask }, TSplWv
				EndFor	
			EndFor
		SetDataFolder root:
			KillWaves Mask, Mask2, MaskPtNumber, TMask
	End
	
	